@@ -2,6 +2,6 @@ mod general;
 mod queries;
 mod tools;
 
-pub use general::{Alias, Aliasable, Column, Expression};
+pub use general::{with, Alias, Aliasable, Column, Expression};
 pub use queries::{delete_from, insert_into, select, update, Joinable, Orderable, Select};
 pub use tools::Parameters;
