@@ -2,25 +2,36 @@ use std::fmt::{self, Display, Formatter};
 
 use itertools::Itertools;
 
-use crate::postgres::general::{Condition, OutputExpression, TableName};
+use crate::postgres::general::{Condition, OutputExpression, TableName, WithClause};
 use crate::tools::IntoIteratorOfSameType;
 
-#[must_use = "Making a query without using it pointless"]
 pub fn delete_from(table_name: impl Into<TableName>) -> DeleteFrom {
-    DeleteFrom {
-        table_name: table_name.into(),
-        ..Default::default()
-    }
+    DeleteFrom::new(table_name.into(), None)
 }
 
-#[derive(Default, Debug)]
+pub(crate) fn delete_from_with(table_name: TableName, with: WithClause) -> DeleteFrom {
+    DeleteFrom::new(table_name, Some(with))
+}
+
+#[must_use = "Making a DELETE FROM without using it is pointless"]
+#[derive(Debug, Clone)]
 pub struct DeleteFrom {
     table_name: TableName,
+    with: Option<WithClause>,
     where_: Vec<Condition>,
     returning: Vec<OutputExpression>,
 }
 
 impl DeleteFrom {
+    fn new(table_name: TableName, with: Option<WithClause>) -> DeleteFrom {
+        DeleteFrom {
+            table_name,
+            with,
+            where_: Vec::new(),
+            returning: Vec::new(),
+        }
+    }
+
     pub fn where_(mut self, conditions: impl IntoIteratorOfSameType<Condition>) -> Self {
         self.where_.extend(conditions.into_some_iter());
         self
@@ -34,6 +45,10 @@ impl DeleteFrom {
 
 impl Display for DeleteFrom {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        if let Some(with_clause) = &self.with {
+            write!(f, "{} ", with_clause)?;
+        }
+
         write!(f, "DELETE FROM {}", self.table_name,)?;
 
         if self.where_.len() > 0 {
@@ -50,8 +65,8 @@ impl Display for DeleteFrom {
 
 #[cfg(test)]
 mod tests {
-    use crate::postgres::delete_from;
     use crate::postgres::tools::tests::assert_correct_postgresql;
+    use crate::postgres::{delete_from, select, with};
 
     #[test]
     fn everything() {
@@ -92,4 +107,14 @@ mod tests {
         let sql = delete_from("Dummy").returning(("id", "place")).to_string();
         assert_correct_postgresql(&sql, "DELETE FROM Dummy RETURNING id, place");
     }
+
+    #[test]
+    fn cte() {
+        let sql = with("thing")
+            .as_(select("1 + 1"))
+            .delete_from("Dummy")
+            .to_string();
+
+        assert_correct_postgresql(&sql, "WITH thing AS (SELECT 1 + 1) DELETE FROM Dummy");
+    }
 }
