@@ -4,32 +4,37 @@ use std::fmt::{self, Display, Formatter};
 
 use itertools::Itertools;
 
-use crate::postgres::general::{Column, Expression, OutputExpression, TableName};
+use crate::postgres::general::{Column, Expression, OutputExpression, TableName, WithClause};
 use crate::tools::{IntoIteratorOfSameType, IntoNonZeroArray};
 
 pub use values::{DefaultValues, Values, WithColumns, WithoutColumns};
 
-#[must_use = "Making a query without using it pointless"]
 pub fn insert_into(table_name: impl Into<TableName>) -> BareInsertInto {
     BareInsertInto {
         table_name: table_name.into(),
+        with: None,
+    }
+}
+
+pub(crate) fn insert_into_with(table_name: TableName, with: WithClause) -> BareInsertInto {
+    BareInsertInto {
+        table_name,
+        with: Some(with),
     }
 }
 
 /* Initial INSERT INTO statement without a valid values clause specified */
 
+#[must_use = "Making a bare INSERT INTO query is pointless"]
 #[derive(Debug)]
 pub struct BareInsertInto {
     table_name: TableName,
+    with: Option<WithClause>,
 }
 
 impl BareInsertInto {
     pub fn default_values(self) -> InsertInto<DefaultValues> {
-        InsertInto {
-            table_name: self.table_name,
-            values: DefaultValues,
-            returning: Vec::new(),
-        }
+        InsertInto::new(self.table_name, DefaultValues, self.with)
     }
 
     pub fn values<T: IntoNonZeroArray<Expression, N>, const N: usize>(
@@ -41,11 +46,7 @@ impl BareInsertInto {
             .map(IntoNonZeroArray::into_non_zero_array)
             .collect();
 
-        InsertInto {
-            table_name: self.table_name,
-            values: WithoutColumns::new(values),
-            returning: Vec::new(),
-        }
+        InsertInto::new(self.table_name, WithoutColumns::new(values), self.with)
     }
 
     pub fn columns<const N: usize>(
@@ -54,6 +55,7 @@ impl BareInsertInto {
     ) -> InsertIntoColumnsBuilder<N> {
         InsertIntoColumnsBuilder {
             table_name: self.table_name,
+            with: self.with,
             columns: columns.into_non_zero_array(),
         }
     }
@@ -61,9 +63,11 @@ impl BareInsertInto {
 
 /* Intermediate struct to ensure one cannot build an INSERT INTO statement with columns, but without values */
 
+#[must_use = "Making a bare INSERT INTO query with columns is pointless"]
 #[derive(Debug)]
 pub struct InsertIntoColumnsBuilder<const N: usize> {
     table_name: TableName,
+    with: Option<WithClause>,
     columns: [Column; N],
 }
 
@@ -72,27 +76,40 @@ impl<const N: usize> InsertIntoColumnsBuilder<N> {
         self,
         values: impl IntoIterator<Item = T>,
     ) -> InsertInto<WithColumns<N>> {
-        let mut values_with_columns = WithColumns::new(self.columns);
-        values_with_columns.add(values);
+        let values = values
+            .into_iter()
+            .map(IntoNonZeroArray::into_non_zero_array)
+            .collect();
 
-        InsertInto {
-            table_name: self.table_name,
-            values: values_with_columns,
-            returning: Vec::new(),
-        }
+        InsertInto::new(
+            self.table_name,
+            WithColumns::new(self.columns, values),
+            self.with,
+        )
     }
 }
 
 /* A valid INSERT INTO statement that can already be stringified */
 
-#[derive(Debug)]
+#[must_use = "Making an INSERT INTO query without using it is pointless"]
+#[derive(Debug, Clone)]
 pub struct InsertInto<V: Values> {
     table_name: TableName,
+    with: Option<WithClause>,
     values: V,
     returning: Vec<OutputExpression>,
 }
 
 impl<V: Values> InsertInto<V> {
+    fn new(table_name: TableName, values: V, with: Option<WithClause>) -> InsertInto<V> {
+        InsertInto {
+            table_name,
+            with,
+            values,
+            returning: Vec::new(),
+        }
+    }
+
     pub fn returning(mut self, expressions: impl IntoIteratorOfSameType<OutputExpression>) -> Self {
         self.returning.extend(expressions.into_some_iter());
         self
@@ -121,6 +138,10 @@ impl<const N: usize> InsertInto<WithoutColumns<N>> {
 
 impl<V: Values> Display for InsertInto<V> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        if let Some(with_clause) = &self.with {
+            write!(f, "{} ", with_clause)?;
+        }
+
         write!(f, "INSERT INTO {} {}", self.table_name, self.values)?;
 
         if self.returning.len() > 0 {
@@ -133,8 +154,8 @@ impl<V: Values> Display for InsertInto<V> {
 
 #[cfg(test)]
 mod tests {
-    use crate::postgres::insert_into;
     use crate::postgres::tools::tests::assert_correct_postgresql;
+    use crate::postgres::{insert_into, select, with};
 
     #[test]
     fn default_values() {
@@ -247,4 +268,18 @@ mod tests {
             "INSERT INTO Dummy (col1) VALUES (a) RETURNING id, place",
         );
     }
+
+    #[test]
+    fn cte() {
+        let sql = with("thing")
+            .as_(select("1 + 1"))
+            .insert_into("Dummy")
+            .values(["a"])
+            .to_string();
+
+        assert_correct_postgresql(
+            &sql,
+            "WITH thing AS (SELECT 1 + 1) INSERT INTO Dummy VALUES (a)",
+        );
+    }
 }
