@@ -16,7 +16,6 @@ pub use from_item::FromItem;
 pub use join::Joinable;
 pub use order_by::{OrderBy, Orderable};
 
-#[must_use = "Making a query without using it pointless"]
 pub fn select(expressions: impl IntoIteratorOfSameType<Expression>) -> Select {
     Select {
         expressions: expressions.into_some_iter().collect(),
@@ -24,18 +23,16 @@ pub fn select(expressions: impl IntoIteratorOfSameType<Expression>) -> Select {
     }
 }
 
-pub(crate) fn select_with(
-    expressions: impl IntoIteratorOfSameType<Expression>,
-    with_clause: WithClause,
-) -> Select {
+pub(crate) fn select_with(expressions: Vec<Expression>, with_clause: WithClause) -> Select {
     Select {
-        expressions: expressions.into_some_iter().collect(),
+        expressions,
         with: Some(with_clause),
         ..Default::default()
     }
 }
 
-#[derive(Default, Debug)]
+#[must_use = "Making a SELECT statement without using it is pointless"]
+#[derive(Default, Debug, Clone)]
 pub struct Select {
     with: Option<WithClause>,
     expressions: Vec<Expression>,
