@@ -2,43 +2,67 @@ use std::fmt::{self, Display, Formatter};
 
 use itertools::Itertools;
 
-use crate::postgres::general::{Column, Condition, Expression, OutputExpression, TableName};
+use crate::postgres::general::{
+    Column, Condition, Expression, OutputExpression, TableName, WithClause,
+};
 use crate::tools::IntoIteratorOfSameType;
 
-#[must_use = "Making a query without using it pointless"]
-pub fn update(table_name: impl Into<TableName>) -> UpdateWithoutAnyValuesSet {
-    UpdateWithoutAnyValuesSet {
+pub fn update(table_name: impl Into<TableName>) -> BareUpdate {
+    BareUpdate {
         table_name: table_name.into(),
+        with: None,
     }
 }
 
+pub(crate) fn update_with(table_name: TableName, with: WithClause) -> BareUpdate {
+    BareUpdate {
+        table_name,
+        with: Some(with),
+    }
+}
+
+#[must_use = "Making an UPDATE query with no values set is pointless"]
 #[derive(Debug)]
-pub struct UpdateWithoutAnyValuesSet {
+pub struct BareUpdate {
     table_name: TableName,
+    with: Option<WithClause>,
 }
 
-impl UpdateWithoutAnyValuesSet {
+impl BareUpdate {
     pub fn set(self, column: impl Into<Column>, value: impl Into<Expression>) -> Update {
-        let mut values = Vec::new();
-        values.push((column.into(), value.into()));
-
-        Update {
-            table_name: self.table_name,
-            values,
-            ..Default::default()
-        }
+        Update::new(
+            self.table_name,
+            vec![(column.into(), value.into())],
+            self.with,
+        )
     }
 }
 
-#[derive(Debug, Default)]
+#[must_use = "Making an UPDATE query without using it is pointless"]
+#[derive(Debug, Clone)]
 pub struct Update {
     table_name: TableName,
+    with: Option<WithClause>,
     values: Vec<(Column, Expression)>,
     where_: Vec<Condition>,
     returning: Vec<OutputExpression>,
 }
 
 impl Update {
+    fn new(
+        table_name: TableName,
+        values: Vec<(Column, Expression)>,
+        with: Option<WithClause>,
+    ) -> Update {
+        Update {
+            table_name,
+            values,
+            with,
+            where_: Vec::new(),
+            returning: Vec::new(),
+        }
+    }
+
     pub fn set(mut self, column: impl Into<Column>, value: impl Into<Expression>) -> Self {
         self.values.push((column.into(), value.into()));
         self
@@ -57,6 +81,10 @@ impl Update {
 
 impl Display for Update {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        if let Some(with_clause) = &self.with {
+            write!(f, "{} ", with_clause)?;
+        }
+
         write!(
             f,
             "UPDATE {} SET {}",
@@ -82,7 +110,7 @@ impl Display for Update {
 #[cfg(test)]
 mod tests {
     use crate::postgres::tools::tests::assert_correct_postgresql;
-    use crate::postgres::update;
+    use crate::postgres::{select, update, with};
 
     #[test]
     fn update_single_value() {
@@ -107,4 +135,15 @@ mod tests {
         let sql = update("Dummy").set("x", "y").returning("x").to_string();
         assert_correct_postgresql(&sql, "UPDATE Dummy SET x = y RETURNING x");
     }
+
+    #[test]
+    fn cte() {
+        let sql = with("thing")
+            .as_(select("1 + 1"))
+            .update("Dummy")
+            .set("x", "y")
+            .to_string();
+
+        assert_correct_postgresql(&sql, "WITH thing AS (SELECT 1 + 1) UPDATE Dummy SET x = y");
+    }
 }
